
#include "NIFTIFloatImage.h"


/*
 Color for Publication 
 <colour index="0" label="Unknown">0,0,0,0</colour>
 <colour index="1" label="FrontalCortex">0,0,205,0</colour>
 <colour index="2" label="TemporalCortex">0,200,0,0</colour>
 <colour index="3" label="MotorCortexRight">255,20,0,0</colour>
 <colour index="4" label="SensoryCortexRight">135,206,250,0</colour>
 <colour index="5" label="PosteriorParietalCortex">235,180,0,0</colour>
 <colour index="6" label="OccipitalCortex">255,255,0,0</colour>
*/ 



double ColorFrontalCortexLeft[3]={0,0,0.8};

double ColorTemporalCortexLeft[3]={0,0.78,0};

double ColorMotorCortexLeft[3]={1,0.078,0};

double ColorSensoryCortexLeft[3]={0.52,0.81,0.98};

double ColorPosteriorParietalCortexLeft[3]={0.92,0.71,0};

double ColorOccipitalCortexLeft[3]={1,1,0};






//DTI Data Management 
template<typename T> unsigned char*  GetScaledData(T* dataPointer, int numberOfDataEl)
{
	
	unsigned char *Udata= new unsigned char[numberOfDataEl];
	
	T *NiftiData=(T*) dataPointer;  
	T max=NiftiData[0];
	T min=NiftiData[1];
	
	for (int i=0;i<numberOfDataEl;i++)
	{
		//cout << NiftiData[i] << " " << endl;
		if (NiftiData[i]>max) max=NiftiData[i];
		if (NiftiData[i]<min) min=NiftiData[i];
	};
	//cout << "Max=" << max << endl;
	//cout << "Min=" << min << endl;
	for (int i=0;i<numberOfDataEl;i++)
	{
		double preCast=NiftiData[i]*255.0/max;
		Udata[i]=(unsigned char) preCast;
	};
	//cout << "Done." << endl; 
	return Udata;
	
};






template<typename T> float*  GetFloatData(T* dataPointer, int numberOfDataEl)
{
	
	float *FloatData= new float[numberOfDataEl];
	
	T *NiftiData=(T*) dataPointer;  
	
	for (int i=0;i<numberOfDataEl;i++)
	{
		float preCast=NiftiData[i];
		FloatData[i]=preCast;
	};
	//cout << "Done." << endl; 
	return FloatData;
	
};



NIFTIFloatImage::
~NIFTIFloatImage()
{
	
	if (nim!=NULL) nifti_image_free(nim);
	if (ImageData!=NULL) delete [] ImageData;
	if (Udata!=NULL) delete [] Udata;
};

NIFTIFloatImage::
NIFTIFloatImage()
{
	sx=0;
	sy=0;
	sz=0;
	numberOfDataEl=0;
	dx=0;
	dy=0;
	dz=0;
	ImageData= NULL;
	Udata= NULL;
	nim=NULL;
};

NIFTIFloatImage::
NIFTIFloatImage(NIFTIIntImage *source)
{
	sx=source->sx;
	sy=source->sy;
	sz=source->sz;
	numberOfDataEl=sx*sy*sz;
	dx=source->dx;
	dy=source->dy;
	dz=source->dz;
	ImageData= new float  [numberOfDataEl];
	Udata=NULL;
	nbyper=0;
	nim=NULL;
	for (int i=0;i<numberOfDataEl;i++)
	{
		ImageData[i]=0;
	};
	
};


NIFTIFloatImage::
NIFTIFloatImage(NIFTIFloatImage *source)
{
	sx=source->sx;
	sy=source->sy;
	sz=source->sz;
	numberOfDataEl=sx*sy*sz;
	dx=source->dx;
	dy=source->dy;
	dz=source->dz;
	ImageData= new float  [numberOfDataEl];
	Udata=NULL;
	nbyper=0;
	nim=NULL;
	for (int i=0;i<numberOfDataEl;i++)
	{
		ImageData[i]=0;
	};
	
};

NIFTIFloatImage::
NIFTIFloatImage(int size[3], double d[3])
{
	sx=size[0];
	sy=size[1];
	sz=size[2];
	numberOfDataEl=sx*sy*sz;
	dx=d[0];
	dy=d[1];
	dz=d[2];
	ImageData= new float [numberOfDataEl];
	Udata=new unsigned char [numberOfDataEl];
	nbyper=0;
	nim=NULL;
	for (int i=0;i<numberOfDataEl;i++)
	{
		ImageData[i]=0.0;
		Udata[i]=0;
	};

	
};




int 
NIFTIFloatImage::
ConvertIJKtoXYZ(int i, int j, int k, float &x, float &y, float &z)
{
if (nim->qform_code==0 && nim->sform_code==0 )
	{
	x = (nim->pixdim[1]) * i;
     	y = (nim->pixdim[2]) * j;
     	z = (nim->pixdim[3]) * k;
	return 0;
	};	
  float *srow_x=nim->sto_xyz.m[0];
  float *srow_y=nim->sto_xyz.m[1];
  float *srow_z=nim->sto_xyz.m[2];
  x = srow_x[0] * i + srow_x[1] * j + srow_x[2] * k + srow_x[3];
  y = srow_y[0] * i + srow_y[1] * j + srow_y[2] * k + srow_y[3];
  z = srow_z[0] * i + srow_z[1] * j + srow_z[2] * k + srow_z[3];
  return 0;
};


//Remark ijk are voxel coordinates whereas xyz are reference space coordinates 
int 
NIFTIFloatImage::
ConvertXYZtoIJK(float x, float y, float z, int &i, int &j, int &k)
{ 
if (nim->qform_code==0 && nim->sform_code==0 )
	{
	float hI=x /(nim->pixdim[1]) ;
     	float hJ= y / (nim->pixdim[2]) ;
     	float hK=z / (nim->pixdim[3]) ;
	i=(int) hI;
	j=(int) hJ;
  	k=(int) hK;
	return 0;
	};
  float *srow_i=nim->sto_ijk.m[0];
  float *srow_j=nim->sto_ijk.m[1];
  float *srow_k=nim->sto_ijk.m[2];
  float iF = srow_i[0] * x + srow_i[1] * y + srow_i[2] * z + srow_i[3];
  float jF = srow_j[0] * x + srow_j[1] * y + srow_j[2] * z + srow_j[3];
  float kF = srow_k[0] * x + srow_k[1] * y + srow_k[2] * z + srow_k[3];
  i=(int) iF;
  j=(int) jF;
  k=(int) kF;
  return 0;
};





float
NIFTIFloatImage::
Max()
{
	float max=ImageData[0];
	float min=ImageData[0];
	
	for (int i=0;i<numberOfDataEl;i++)
	{
		if (ImageData[i]>max) max=	ImageData[i];
		if (ImageData[i]<min) min=	ImageData[i];
	};
	
	return max;
}





int
NIFTIFloatImage::
PrintMinMaxNrNonZero()
{
	int min=ImageData[0];
	int max=ImageData[0];
	int NrNonZero=0;
	
	
	for (int i=0;i<numberOfDataEl;i++)
	{
		if (ImageData[i]>max) max=	ImageData[i];
		if (ImageData[i]<min) min=	ImageData[i];
		if (ImageData[i]!=0) NrNonZero++;
	};
	
	cout << "Min:" << min << endl;
	cout << "Max:" << max << endl;
	cout << "NonZero:" << NrNonZero << "/" <<  numberOfDataEl << endl;
	
	return 0;
};


int
NIFTIFloatImage::
DoDimMatch(NIFTIFloatImage *source)
{
	if (sx!=source->sx) return 0;
	if (sy!=source->sy) return 0;
	if (sz!=source->sz) return 0;
	return 1;
	
};




NIFTIFloatImage::
NIFTIFloatImage(char *directory, int nr)
{

};



int 
NIFTIFloatImage::
SetVoxel(int x, int y, int z, float value)
{
	int i=x+y*sx+z*sx*sy;
	if (x<0 || x>=sx || y<0 || y>=sy || z<0 || z>=sz) return 1; 
	ImageData[i]=value;
	return 0;
};



float 
NIFTIFloatImage::
GetVoxel(int x, int y, int z)
{
	int i=x+y*sx+z*sx*sy;
	float value=ImageData[i];
				
	return value;
	

};


int 
NIFTIFloatImage::
CopyDataFrom(NIFTIFloatImage *source)
{
	if (DoDimMatch(source)==0) return 1;
	for (int i=0;i<numberOfDataEl;i++)
	{
		ImageData[i]=(source->ImageData)[i];
	};
	
	return 0;
};





int  
NIFTIFloatImage::
LoadNiftiImage(const char *fin)
{
	if (nim!=NULL) nifti_image_free(nim);
	nim=NULL;
	/* read input dataset, including data */
	nim = nifti_image_read(fin, 1);
	if( !nim ) {
		cout << "** failed to read NIfTI image from " << fin << endl;
		return 1;
	}
	

	sx=nim->nx;
	sy=nim->ny;
	sz=nim->nz;
	dx=nim->dx;
	dy=nim->dy;
	dz=nim->dz;
	nbyper =nim->nbyper; //number of bytes per voxel;
	numberOfDataEl=sx*sy*sz;
	//cout << "Number of data el:" << numberOfDataEl << endl;
	//cout << "sx:" << sx << " " << "sy:" << sy << " " << "sz:" << sz << " " << endl;
	//cout << "dx:" << dx << " " << "dy:" << dy << " " << "dz:" << dz << " " << endl;
	//cout << "Number of bytes per voxel (nbyper):" << nbyper << endl;
	
	if (0)
	{
	//NIFTI DATA TYPE
	if (Udata!=NULL) delete [] Udata;
	Udata= new unsigned char[sx*sy*sz];
	switch (nim->datatype)
	{	  
		case DT_BINARY                   :  cout << "  binary (1 bit/voxel)         " << endl; return 1;break;
		case DT_UNSIGNED_CHAR            :  cout << " unsigned char (8 bits/voxel) " << endl; Udata=GetScaledData((unsigned char*) nim->data, numberOfDataEl); break;
		case DT_SIGNED_SHORT             :  cout << " signed short (16 bits/voxel) " << endl; Udata=GetScaledData((signed short*) nim->data, numberOfDataEl); break;
		case DT_SIGNED_INT               :  cout << " signed int (32 bits/voxel)   " << endl; Udata=GetScaledData((signed int*) nim->data, numberOfDataEl); break;
		case DT_FLOAT                    :  cout << " float (32 bits/voxel)        " << endl; Udata=GetScaledData((float*) nim->data, numberOfDataEl); break;
		case DT_COMPLEX                  :   cout << " complex (64 bits/voxel)      " << endl; return 1;break;
		case DT_DOUBLE                   :  cout << " double (64 bits/voxel)       " << endl;Udata=GetScaledData((double*) nim->data, numberOfDataEl); break;
		case DT_RGB                      :  cout << " RGB triple (24 bits/voxel)   " << endl; return 1;break;
		default : 	cout << "Unknown data type" << endl; exit(1); break;
	};
	};
	
	
	if (ImageData!=NULL) delete [] ImageData;
	//ImageData= new float[numberOfDataEl];
	switch (nim->datatype)
	{	  
		case DT_BINARY                   :  cout << "  binary (1 bit/voxel)         " << endl; return 1;break;
		case DT_UNSIGNED_CHAR            :  cout << " unsigned char (8 bits/voxel) " << endl; ImageData=GetFloatData((unsigned char*) nim->data, numberOfDataEl); break;
		case DT_SIGNED_SHORT             :  cout << " signed short (16 bits/voxel) " << endl; ImageData=GetFloatData((signed short*) nim->data, numberOfDataEl); break;
		case DT_SIGNED_INT               :  cout << " signed int (32 bits/voxel)   " << endl; ImageData=GetFloatData((signed int*) nim->data, numberOfDataEl); break;
		case DT_FLOAT                    :  cout << " float (32 bits/voxel)        " << endl; ImageData=GetFloatData((float*) nim->data, numberOfDataEl); break;
		case DT_COMPLEX                  :   cout << " complex (64 bits/voxel)      " << endl; return NULL;break;
		case DT_DOUBLE                   :  cout << " double (64 bits/voxel)       " << endl; ImageData=GetFloatData((double*) nim->data, numberOfDataEl); break;
		case DT_RGB                      :  cout << " RGB triple (24 bits/voxel)   " << endl; return 1;break;
		default : 	cout << "Unknown data type" << endl; exit(1); break;
	};
	
	
	
	
		
	return 0;

};






int  
NIFTIFloatImage::
WriteNiftiImage(const char *fin)
{
	
	sprintf(FileName,"%s",fin);
	
	if (nim->datatype!=DT_FLOAT)
	{
		cout << "This is not a float 32 bits per voxel image" << endl;
		cout << "float (32 bits/voxel) " << endl;
		exit (1);
	};
	
	
	//case DT_SIGNED_INT               :  cout << " signed int (32 bits/voxel)   " << endl; ImageData=GetIntData((signed int*) nim->data, numberOfDataEl); break;
	
	float *Pointer=(float*) nim->data;
	for (int i=0;i<numberOfDataEl;i++) Pointer[i]=ImageData[i];
	nifti_set_filenames(nim, FileName, 0, nim->byteorder);
	nifti_image_write   ( nim) ;
	return 0;
};







int  
NIFTIFloatImage::
WriteNonFloatNiftiImage(const char *fin)
{
	
	sprintf(FileName,"%s",fin);

	
	float *FloatP;
	double *DoubleP;
	int *SignedIntP;
	unsigned char *UCharP;
	float maxValue=Max();

	float *Pointer=(float*) nim->data;
	for (int i=0;i<numberOfDataEl;i++) Pointer[i]=ImageData[i];


	switch (nim->datatype)
	{	  
	
		case DT_UNSIGNED_CHAR            :  
		  cout << " unsigned char (8 bits/voxel) not implemented  " << endl;
		  return 1;
		  //ImageData=GetFloatData((unsigned char*) nim->data, numberOfDataEl); 
		  break;

	
		case DT_SIGNED_INT               :  
		  cout << " signed int (32 bits/voxel)   " << endl; 
		  SignedIntP=(int*) nim->data;
		  for (int i=0;i<numberOfDataEl;i++) SignedIntP[i]=ImageData[i];
		  break;

		case DT_FLOAT                    :  
		  cout << " float (32 bits/voxel)        " << endl; 
		  FloatP=(float*) nim->data;
		  for (int i=0;i<numberOfDataEl;i++) FloatP[i]=ImageData[i];

		  
		  break;
	
		case DT_DOUBLE                   :  
		  cout << " double (64 bits/voxel)       " << endl; 
		   DoubleP=(double*) nim->data;
		  for (int i=0;i<numberOfDataEl;i++) DoubleP[i]=ImageData[i]; 
		  break;
	
		default : 	cout << "Unknown data type" << endl; exit(1); break;
	};
	





	nifti_set_filenames(nim, FileName, 0, nim->byteorder);
	nifti_image_write   ( nim) ;
	return 0;
};










int  
NIFTIFloatImage::
WriteRawDataToDisk(const char* fileName)
{	
  char name[100];
//  sprintf(name, "%s", fileName);
  cout << "Write Raw Data" << endl;
  //cout << "Size of uint:" << sizeof( unsigned short) << endl;
   cout << "Size of uint:" << sizeof( unsigned char) << endl;
  
  cout << "sx: " << sx << " sy: " << sy << " sz: " << sz << endl;
  cout << "dx: " << dx << " dy: " << dy << " dz: " << dz << endl;
  cout << "Field of View: x(mm)" << sx*dx << " y:" << sy*dy << " Slices Thickness: " << dz << endl;  

  cout << "Write To File:" << fileName << endl;
  cout << "For MRConvert: View Dir Axial, Flip vertical, Invert slice order!" << endl;
  cout << "Rows:" << sy << endl;
  cout << "Columns" << sx << endl;

  unsigned char *buffer= new unsigned char[numberOfDataEl]; 
  FILE *file=fopen(fileName,"wb");
  float m=Max();
  for (int i=0;i<numberOfDataEl;i++) 
    {
      float temp= ImageData[i]/m*256.0;
      buffer[i]=(unsigned char) temp;
     };
  
  fwrite(buffer,sizeof( unsigned char),numberOfDataEl, file);
  fclose(file);
  delete [] buffer;
  return 0;
};






int 
NIFTIFloatImage::
WriteRawDataToDiskYZX(const char *fileName)
{	
  char name[100];
//  sprintf(name, "%s", fileName);
  cout << "Write Raw Data XYZ swapped to YZX" << endl;
  //cout << "Size of uint:" << sizeof( unsigned short) << endl;
   cout << "Size of uint:" << sizeof( unsigned char) << endl;
  
  cout << "sx: " << sx << " sy: " << sy << " sz: " << sz << endl;
  cout << "dx: " << dx << " dy: " << dy << " dz: " << dz << endl;
  cout << "Field of View: x(mm)" << sx*dx << " y:" << sy*dy << " Slices Thickness: " << dz << endl;  

  cout << "Write To File:" << fileName << endl;
  cout << "For MRConvert: 8bit" << endl;
  cout << "Nr of images:" << sx << endl;
  cout << "Rows:" <<  sz << endl;
  cout << "Columns" << sy << endl;

  cout << "Field of view " << endl;
  cout << dz*sz << " High" << endl;
  cout << dy*sy << " Wide" << endl;
  cout << "Slice Thickness:" << dx << endl;


  unsigned char *buffer= new unsigned char[numberOfDataEl]; 
  FILE *file=fopen(fileName,"wb");
  float m=Max();
  int i=0;

  for(int x=0;x<sx;x++)
    for(int z=(sz-1);z>=0;z--)
      for(int y=(sy-1);y>=0;y--)
	  {
	    float temp= GetVoxel(x,y,z)/m*256.0;
		   buffer[i]=(unsigned char) temp;
		   i++;
		 };	  




  





  fwrite(buffer,sizeof( unsigned char),numberOfDataEl, file);
  fclose(file);
  delete [] buffer;
  return 0;
};


































vtkImageData*
NIFTIFloatImage::
GetVTKImageData()
{
	Udata=GetScaledData(ImageData, numberOfDataEl);
	
	//Nifti Data
	vtkUnsignedCharArray *array=vtkUnsignedCharArray::New();
	array->SetArray(Udata, sx*sy*sz,1 );
	
	vtkImageData *imageData=vtkImageData::New();
	(imageData->GetPointData())->SetScalars(array);
	imageData->SetDimensions (sx, sy, sz);
	imageData->SetSpacing	(	dx 	, dy 	, dz);	
	imageData->SetScalarType(VTK_UNSIGNED_CHAR);
	imageData->SetOrigin(0.0,0.0,0.0);
	
	
	return imageData;
}



// Nifti Image Processing 





