//Nifti Image Management 

#ifndef NIFTIFLOATIMAGE
#define NIFTIFLOATIMAGE

#include "DataTypes.h"
#include "NIFTIIntImage.h"

#define FRONTALCORTEXLEFT 1
#define MOTORCORTEXLEFT 2
#define OCCIPITALCORTEXLEFT 3
#define POSTERIORPARIETALCORTEXLEFT 4
#define SENSORYCORTEXLEFT 5
#define TEMPORALCORETXLEFT 6

extern double ColorFrontalCortexLeft[3];
extern double ColorTemporalCortexLeft[3];
extern double ColorMotorCortexLeft[3];
extern double ColorSensoryCortexLeft[3];
extern double ColorPosteriorParietalCortexLeft[3];
extern double ColorOccipitalCortexLeft[3];





template<typename T> unsigned char*  GetScaledData(T* dataPointer, int numberOfDataEl);
template<typename T> float*  GetFloatData(T* dataPointer, int numberOfDataEl);



class NIFTIFloatImage
{
public:
	NIFTIFloatImage(NIFTIIntImage *source);
	NIFTIFloatImage(NIFTIFloatImage *source);
	
	~NIFTIFloatImage();
	NIFTIFloatImage();
	NIFTIFloatImage(int size[3], double  d[3]);
	NIFTIFloatImage(char *directory, int nr);
	int LoadNiftiImage(const char *fin);
	
	float GetVoxel(int x, int y, int z);
	int SetVoxel(int x, int y, int z, float value);
	float Max();
	
	float GetPixel(int x, int y, int z) {return GetVoxel(x, y, z);};
	int SetPixel(int x, int y, int z, float value) {return SetVoxel(x, y, z, value);};
	int ConvertIJKtoXYZ(int i, int j, int k, float &x, float &y, float &z);
	int ConvertXYZtoIJK(float x, float y, float z, int &i, int &j, int &k);


	vtkImageData* GetVTKImageData();

	int CopyDataFrom(NIFTIFloatImage *source);
	
	
	int 	DoDimMatch(NIFTIFloatImage *source);
	int 	PrintMinMaxNrNonZero();
	
	int  WriteNiftiImage(const char *fin);
	int  WriteNonFloatNiftiImage(const char *fin);
	int  WriteRawDataToDisk(const char *fileName);
	int WriteRawDataToDiskYZX(const char *fileName);

public:
	int sx;
	int sy;
	int sz;
	double dx;
	double dy;
	double dz;
	int numberOfDataEl;
	nifti_image * nim; // does not need to exist 
	int nbyper;
	unsigned char *Udata;
	float *ImageData;


	//ColorManagement 
	double  color[3];
	
	//FileName
	char FileName[120];
	char BaseName[120];
	
};


#endif


