/*
 *  VTKVisualization.cpp
 *  
 *
 *  Created by Philip Broser on 5/13/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */

#include "VTKVisualization.h"
#include "MNIvtkCubeAxesActor2D.h"

#include <vtkDelaunay3D.h>
#include <vtkDataSetSurfaceFilter.h>
#include <vtkSmoothPolyDataFilter.h>
#include <vtkButterflySubdivisionFilter.h>
#include <vtkTubeFilter.h>
#include <vtkLineSource.h>


vtkActor *GetGlyphActorWithDelaunyForPolyData(vtkPolyData *polydata, int mode)
{
	vtkDelaunay3D* delaunay = vtkDelaunay3D::New();
	delaunay->AddInput(polydata);
	
	if (mode==0)
	  {
	    delaunay->SetTolerance(0.0);
	    delaunay->SetAlpha(6);
	    //delaunay->BoundingTriangulationOff();
	  };




delaunay->Update();
	vtkUnstructuredGrid* GridDatas=delaunay->GetOutput();
	
	
	vtkDataSetSurfaceFilter* surface=vtkDataSetSurfaceFilter::New();
 	surface->SetInput(GridDatas);
	
    vtkPolyDataMapper *mapper=vtkPolyDataMapper::New();

/*
    mapper->SetInputConnection(0, surface->GetOutputPort(0));
	*/

    vtkButterflySubdivisionFilter* interPolater=vtkButterflySubdivisionFilter::New();
    interPolater->SetInputConnection(surface->GetOutputPort(0));
    interPolater->SetNumberOfSubdivisions(3);


    vtkSmoothPolyDataFilter* smoother = vtkSmoothPolyDataFilter::New();

      smoother->SetInputConnection(interPolater->GetOutputPort(0));

    smoother->SetNumberOfIterations(2);

    if (mode>0) mapper->SetInputConnection(0, smoother->GetOutputPort(0));
    if (mode==0) mapper->SetInputConnection(0, surface->GetOutputPort(0));


    vtkActor* actor = vtkActor::New();
 actor->SetMapper(mapper);
	return actor;


}




//DTI Data Management 
template<typename T> unsigned char*  GetScaledData(T* dataPointer, int numberOfDataEl)
{
	
	unsigned char *Udata= new unsigned char[numberOfDataEl];
	
	T *NiftiData=(T*) dataPointer;  
	T max=NiftiData[0];
	T min=NiftiData[1];
	
	for (int i=0;i<numberOfDataEl;i++)
	{
		//cout << NiftiData[i] << " " << endl;
		if (NiftiData[i]>max) max=NiftiData[i];
		if (NiftiData[i]<min) min=NiftiData[i];
	};
	//cout << "Max=" << max << endl;
	//cout << "Min=" << min << endl;
	for (int i=0;i<numberOfDataEl;i++)
	{
		double preCast=NiftiData[i]*255.0/max;
		Udata[i]=(unsigned char) preCast;
	};
	//cout << "Done." << endl; 
	return Udata;
	
};






template<typename T> float*  GetFloatData(T* dataPointer, int numberOfDataEl)
{
	
	float *FloatData= new float[numberOfDataEl];
	
	T *NiftiData=(T*) dataPointer;  
	
	for (int i=0;i<numberOfDataEl;i++)
	{
		float preCast=NiftiData[i];
		FloatData[i]=preCast;
	};
	//cout << "Done." << endl; 
	return FloatData;
	
};


vtkActor *
GetGlyphActorForPolyData(vtkPolyData *polydata)
{
	float cubeSize=2.0;
	//create anything you want here, we will use a cube for the demo.
	vtkSmartPointer<vtkCubeSource> cubeSource = 
	vtkSmartPointer<vtkCubeSource>::New();
	cubeSource->SetXLength	( 	cubeSize);
	cubeSource->SetYLength	( 	cubeSize);
	cubeSource->SetZLength	( 	cubeSize);
	
	
	
	vtkSmartPointer<vtkGlyph3D> glyph3D = 
	vtkSmartPointer<vtkGlyph3D>::New();
	glyph3D->SetColorModeToColorByScalar();
	glyph3D->SetSource(cubeSource->GetOutput());
	glyph3D->SetInput(polydata);
	glyph3D->ScalingOff();
	glyph3D->Update();
	
	//Create a mapper and actor
	vtkSmartPointer<vtkPolyDataMapper> mapper = 
	vtkSmartPointer<vtkPolyDataMapper>::New();
	mapper->SetInputConnection(glyph3D->GetOutputPort());
	vtkActor* actor = vtkActor::New();
	actor->SetMapper(mapper);
	
	return actor;
}



vtkActor *
GetGlyphActorForPolyData(vtkPolyData *polydata, float cubeSize )
{
	//float cubeSize=2.0;
	//create anything you want here, we will use a cube for the demo.
	vtkSmartPointer<vtkCubeSource> cubeSource = 
	vtkSmartPointer<vtkCubeSource>::New();
	cubeSource->SetXLength	( 	cubeSize);
	cubeSource->SetYLength	( 	cubeSize);
	cubeSource->SetZLength	( 	cubeSize);
	
	
	
	vtkSmartPointer<vtkGlyph3D> glyph3D = 
	vtkSmartPointer<vtkGlyph3D>::New();
	glyph3D->SetColorModeToColorByScalar();
	glyph3D->SetSource(cubeSource->GetOutput());
	glyph3D->SetInput(polydata);
	glyph3D->ScalingOff();
	glyph3D->Update();
	
	//Create a mapper and actor
	vtkSmartPointer<vtkPolyDataMapper> mapper = 
	vtkSmartPointer<vtkPolyDataMapper>::New();
	mapper->SetInputConnection(glyph3D->GetOutputPort());
	vtkActor* actor = vtkActor::New();
	actor->SetMapper(mapper);
	
	return actor;
}


vtkActor *
GetGlyphActorForPolyData(vtkPolyData *polydata, float cubeSize, float Opacity)
{
	//float cubeSize=2.0;
	//create anything you want here, we will use a cube for the demo.
	vtkSmartPointer<vtkCubeSource> cubeSource = 
	vtkSmartPointer<vtkCubeSource>::New();
	cubeSource->SetXLength	( 	cubeSize);
	cubeSource->SetYLength	( 	cubeSize);
	cubeSource->SetZLength	( 	cubeSize);
	
	
	
	vtkSmartPointer<vtkGlyph3D> glyph3D = 
	vtkSmartPointer<vtkGlyph3D>::New();
	glyph3D->SetColorModeToColorByScalar();
	glyph3D->SetSource(cubeSource->GetOutput());
	glyph3D->SetInput(polydata);
	glyph3D->ScalingOff();
	glyph3D->Update();
	
	vtkSmartPointer<vtkPolyDataMapper> mapper = 
	vtkSmartPointer<vtkPolyDataMapper>::New();
	mapper->SetInputConnection(glyph3D->GetOutputPort());
	
	
	
	//Thalamus Outline Visibility:
	
	mapper->ScalarVisibilityOff();
	vtkProperty* pairedProps=vtkProperty::New();
	double r=1.0;
	double g=1.0;
	double b=1.0;
	pairedProps->SetColor(r,g,b);
	pairedProps->SetAmbientColor(r,g,b);
	pairedProps->SetDiffuseColor(r,g,b);
	pairedProps->SetSpecularColor(r,g,b);
	pairedProps->SetOpacity(Opacity);
	
	//Create a mapper and actor
	
	
	
	vtkActor* actor = vtkActor::New();
	actor->SetMapper(mapper);
	actor-> SetProperty (pairedProps);
	return actor;
}









vtkActor* GetActorForPolyDataFileWithColor(const char *fileName, float r, float g, float b, float opacity)
{
	
	//Set Up Mapper and Actor for Barrel Field
	vtkPolyDataMapper *mapper = vtkPolyDataMapper::New();
    vtkActor *actor = vtkActor::New();
	//Reader
	
	vtkPolyDataReader *reader=	vtkPolyDataReader::New();
	reader->SetFileName(fileName);
	vtkPolyData *data=reader->GetOutput();
	mapper->SetInput(data);
    mapper->ScalarVisibilityOff();
	actor->SetMapper(mapper);
	vtkProperty* barrelProps=vtkProperty::New();
	barrelProps->SetColor(r,g,b);
	barrelProps->SetOpacity(opacity);
	actor-> SetProperty (barrelProps);
	
	return actor;
}





vtkActor* VisualiseCoordinateSystem()
{
	
	
	int nrOfDataElements=3;
	
	//create points
	vtkSmartPointer<vtkPoints> points = 
	vtkSmartPointer<vtkPoints>::New();
	//for (int i=0; i<nrOfDataElements;i++) 
	  points->InsertNextPoint(40,10,19);
	  points->InsertNextPoint(44,69,27);
	  points->InsertNextPoint(0,0,0);
	
	
	
	//setup scales
	vtkSmartPointer<vtkFloatArray> scales = 
	vtkSmartPointer<vtkFloatArray>::New();
	scales->SetName("scales");
	
	for (int i=0; i<nrOfDataElements;i++)  scales->InsertNextValue(10.0*i);
    
	//setupdirection
	vtkFloatArray *vectors = vtkFloatArray::New();
	vectors->SetNumberOfComponents(3);
	vectors->SetNumberOfTuples(nrOfDataElements);
	
	

		
		
		
	
		
	float Vec[3];
	double l=10;
	double sizeOfVec=l;
	
	int i=0;
	
		Vec[0]=l;
		Vec[1]=0;
		Vec[2]=0;
		vectors->InsertTuple(i,Vec);
		scales->SetValue (i, sizeOfVec);
	
	i=1;
	Vec[0]=0;
	Vec[1]=l;
	Vec[2]=0;
	vectors->InsertTuple(i,Vec);
	scales->SetValue (i, sizeOfVec);
	
	i=2;
	Vec[0]=0;
	Vec[1]=0;
	Vec[2]=l;
	vectors->InsertTuple(i,Vec);
	scales->SetValue (i, sizeOfVec);
	
	
	
	
	
	
	
	
	
	//combine into a polydata
	vtkSmartPointer<vtkPolyData> polydata = 
	vtkSmartPointer<vtkPolyData>::New();
	polydata->SetPoints(points);
	polydata->GetPointData()->SetScalars(scales);
	polydata->GetPointData()->SetVectors(vectors);
	//create anything you want here, we will use a cube for the demo.
	vtkSmartPointer<vtkCubeSource> cubeSource = 
	vtkSmartPointer<vtkCubeSource>::New();
	vtkArrowSource *arrow=vtkArrowSource::New();  
	
	vtkSmartPointer<vtkGlyph3D> glyph3D = 
	vtkSmartPointer<vtkGlyph3D>::New();
	glyph3D->SetScaleModeToScaleByScalar();
	glyph3D->SetSource(arrow->GetOutput());
	glyph3D->SetInput(polydata);
	glyph3D->Update();
	
	//Create a mapper and actor
	vtkSmartPointer<vtkPolyDataMapper> mapper = 
	vtkSmartPointer<vtkPolyDataMapper>::New();
	mapper->SetInputConnection(glyph3D->GetOutputPort());
	vtkActor *actor =  vtkActor::New();
	actor->SetMapper(mapper);

	
	
	
	return actor;
}









vtkPolyData*
GetPolyDataForMaskWithThresholdAndColor(NIFTIIntImage* BaseImage, float Threshold, unsigned char r,  unsigned char  g, unsigned char b)
// NIFTI Voxel Coordinates are transfered into VTK Coordinates, Carefull not physical coordinates
{
	
	int size[3];
	size[0]=BaseImage->sx;
	size[1]=BaseImage->sy;
	size[2]=BaseImage->sz;
	double d[3];
	d[0]=BaseImage->dx;
	d[1]=BaseImage->dy;
	d[2]=BaseImage->dz;
	
	vtkSmartPointer<vtkPoints> points = 
	vtkSmartPointer<vtkPoints>::New();
	
	vtkSmartPointer<vtkUnsignedCharArray> colors = 
	vtkSmartPointer<vtkUnsignedCharArray>::New();
	
	colors->SetName("colors");
	colors->SetNumberOfComponents(3);
	
	int element_counter=0;
	for(int k=0;k<size[2];k++)
		for(int j=0;j<size[1];j++)
			for(int i=0;i<size[0];i++)
			{
				double FixedColor1[3]={0,1.0,0.0};
				
				if ((BaseImage->GetVoxel(i, j, k))>Threshold) 
				{
					
					element_counter++;
					//Create the topology of the point (a vertex)
					float x,y,z;
					BaseImage->ConvertIJKtoXYZ(i, j, k, x, y, z);

					points->InsertNextPoint(x,y,z);
									
					unsigned char UCharColor[3];
					UCharColor[0]=r;
					UCharColor[1]=g;
					UCharColor[2]=b;
					
					colors->InsertNextTupleValue(UCharColor);
					
				};
				
				
			};
	
	cout << "Number of elements >0:  " << element_counter << endl;
	//combine into a polydata
	vtkPolyData *polydata =	vtkPolyData::New();
	polydata->SetPoints(points);
	polydata->GetPointData()->SetScalars(colors);
	
	//Write to a file if req
	int req=1;
	if (req)
	{
		vtkPolyDataWriter *writer=vtkPolyDataWriter::New();
		writer->SetInput (polydata);
		writer->SetFileName	("ThalamusSegregation");
		writer->Update	( 	 	) ;	
		
	};
	
	
	return polydata;
	
};




vtkPolyData* GetPolyDataForFloatImageWithThresholdAndColor(NIFTIFloatImage* BaseImage, float Threshold, unsigned char r, unsigned char g, unsigned char b)
{
	
	int size[3];
	size[0]=BaseImage->sx;
	size[1]=BaseImage->sy;
	size[2]=BaseImage->sz;
	double d[3];
	d[0]=BaseImage->dx;
	d[1]=BaseImage->dy;
	d[2]=BaseImage->dz;
	
	vtkSmartPointer<vtkPoints> points = 
	vtkSmartPointer<vtkPoints>::New();
	
	vtkSmartPointer<vtkUnsignedCharArray> colors = 
	vtkSmartPointer<vtkUnsignedCharArray>::New();
	
	colors->SetName("colors");
	colors->SetNumberOfComponents(3);
	
	int element_counter=0;
	for(int k=0;k<size[2];k++)
		for(int j=0;j<size[1];j++)
			for(int i=0;i<size[0];i++)
			{
				double FixedColor1[3]={0,1.0,0.0};
				
				if ((BaseImage->GetVoxel(i, j, k))>Threshold) 
				{
					
					element_counter++;
					//Create the topology of the point (a vertex)
					float x,y,z;
					BaseImage->ConvertIJKtoXYZ(i, j, k, x, y, z);
					
					points->InsertNextPoint(x,y,z);
					
					unsigned char UCharColor[3];
					UCharColor[0]=r;
					UCharColor[1]=g;
					UCharColor[2]=b;
					
					colors->InsertNextTupleValue(UCharColor);
					
				};
				
				
			};
	
	cout << "Number of elements >0:  " << element_counter << endl;
	//combine into a polydata
	vtkPolyData *polydata =	vtkPolyData::New();
	polydata->SetPoints(points);
	polydata->GetPointData()->SetScalars(colors);
	
	//Write to a file if req
	int req=1;
	if (req)
	{
		vtkPolyDataWriter *writer=vtkPolyDataWriter::New();
		writer->SetInput (polydata);
		writer->SetFileName	("ThalamusSegregation");
		writer->Update	( 	 	) ;	
		
	};
	
	
	return polydata;
	
};









vtkActor *GetActorForPolyDataFileName(char* fileName)
{
	vtkPolyDataReader *NeuronDataReader=vtkPolyDataReader::New();
	
	
	NeuronDataReader->SetFileName(fileName);
	
	NeuronDataReader->Update();
	vtkPolyData *NeuronData= NeuronDataReader->GetOutput();
	vtkPolyDataMapper *NeuronMapper = vtkPolyDataMapper::New();
	vtkActor *NeuronActor = vtkActor::New();
	NeuronMapper->SetInput(NeuronData);
	NeuronActor->SetMapper(NeuronMapper);
	return NeuronActor;
	
}



vtkActor *
GetActorForSmoothPolyDataFileName(char* fileName)
{
	vtkPolyDataReader *NeuronDataReader=vtkPolyDataReader::New();
	
	
	NeuronDataReader->SetFileName(fileName);
	
	NeuronDataReader->Update();
	vtkPolyData *NeuronData= NeuronDataReader->GetOutput();





	vtkButterflySubdivisionFilter* interPolater=vtkButterflySubdivisionFilter::New();
	interPolater->SetInputConnection(NeuronDataReader->GetOutputPort(0));
	interPolater->SetNumberOfSubdivisions(2);


	vtkSmoothPolyDataFilter* smoother = vtkSmoothPolyDataFilter::New();
	smoother->SetInputConnection(interPolater->GetOutputPort(0));
	smoother->SetNumberOfIterations(2);


	vtkPolyDataMapper *NeuronMapper = vtkPolyDataMapper::New();
	vtkActor *NeuronActor = vtkActor::New();
//	NeuronMapper->SetInput(NeuronData);
		NeuronMapper->SetInputConnection(0, smoother->GetOutputPort(0));
	NeuronActor->SetMapper(NeuronMapper);
	return NeuronActor;
	
}










//Load Nifti FA Image

vtkImageData *NiftiIO(char *fin)
{
	nifti_image * nim=NULL;
    
	//char *fin="fa_masked.hdr"; //input sring
	//char *fout="testniftiio.hdr";//outputfilename
    //B0Image.nii.gz
    /* read input dataset, including data */
	nim = nifti_image_read(fin, 1);
	if( !nim ) {
		cout << "** failed to read NIfTI image from " << fin << endl;
		return NULL;
	}
	
	
	
	/* assign nifti_image fname/iname pair, based on output filename
	 (request to 'check' image and 'set_byte_order' here) */
	//if( nifti_set_filenames(nim, fout, 1, 1) ) return 1;
	
	/* if we get here, write the output dataset */
	//nifti_image_write( nim );
	
	
	
	//Create VTK Image 
	int sx=nim->nx;
	int sy=nim->ny;
	int sz=nim->nz;
	double dx=nim->dx;
	double dy=nim->dy;
	double dz=nim->dz;
	int nbyper =nim->nbyper; //number of bytes per voxel;
	int numberOfDataEl=sx*sy*sz;
	cout << "Number of data el:" << numberOfDataEl << endl;
	unsigned char *Udata= new unsigned char[sx*sy*sz];
	cout << "sx:" << sx << " " << "sy:" << sy << " " << "sz:" << sz << " " << endl;
	cout << "dx:" << dx << " " << "dy:" << dy << " " << "dz:" << dz << " " << endl;
	cout << "Number of bytes per voxel (nbyper):" << nbyper << endl;
	
	
	//NIFTI DATA TYPE
	switch (nim->datatype)
	{	  
		case DT_BINARY                   :  cout << "  binary (1 bit/voxel)         " << endl; return NULL;break;
		case DT_UNSIGNED_CHAR            :  cout << " unsigned char (8 bits/voxel) " << endl; Udata=GetScaledData((unsigned char*) nim->data, numberOfDataEl); break;
		case DT_SIGNED_SHORT             :  cout << " signed short (16 bits/voxel) " << endl; Udata=GetScaledData((signed short*) nim->data, numberOfDataEl); break;
		case DT_SIGNED_INT               :  cout << " signed int (32 bits/voxel)   " << endl; Udata=GetScaledData((signed int*) nim->data, numberOfDataEl); break;
		case DT_FLOAT                    :  cout << " float (32 bits/voxel)        " << endl; Udata=GetScaledData((float*) nim->data, numberOfDataEl); break;
		case DT_COMPLEX                  :   cout << " complex (64 bits/voxel)      " << endl; return NULL;break;
		case DT_DOUBLE                   :  cout << " double (64 bits/voxel)       " << endl;Udata=GetScaledData((double*) nim->data, numberOfDataEl); break;
		case DT_RGB                      :  cout << " RGB triple (24 bits/voxel)   " << endl; return NULL;break;
		default : 	cout << "Unknown data type" << endl; exit(1); break;
	};
	
	
	
	
	
	
	
	
	//Nifti Data
	vtkUnsignedCharArray *array=vtkUnsignedCharArray::New();
	array->SetArray(Udata, sx*sy*sz,1 );
	


	//This is not correct as it is
	// one has to use the sform to get the origin and to get the correct orientation
	// against the default coordinate system i.e. phi and theta angle against x,y,z axis
	vtkImageData *imageData=vtkImageData::New();
	(imageData->GetPointData())->SetScalars(array);
	imageData->SetDimensions (sx, sy, sz);
	imageData->SetSpacing	(	dx 	, dy 	, dz);	
	imageData->SetScalarType(VTK_UNSIGNED_CHAR);
	imageData->SetOrigin(0.0,0.0,0.0);
	
	
	
	//GetInfo About Image
	ostringstream oss (ostringstream::out);
	imageData->Print(oss); 
	cout << oss.str() << endl;
	
	
	//Create TiffOutput if req
	vtkTIFFWriter *tiffWriter=vtkTIFFWriter::New(); 
	tiffWriter->SetInput(imageData);
	tiffWriter->SetFileName("rendvol.tif");
	tiffWriter->Write();
	
	
	
	/* and clean up memory */
	nifti_image_free( nim );
	
	
	return imageData;
}










vtkActor* SphereActorForPositionAndColor(float x, float y, float z, float r, float g, float b )
{

 
vtkSphereSource *sphere= vtkSphereSource::New();
sphere->SetRadius(2.0);
vtkPolyDataMapper *sphereMapper= vtkPolyDataMapper::New();
sphereMapper->SetInput(sphere->GetOutput());
vtkActor *sphereActor=vtkActor::New();
sphereActor->SetMapper(sphereMapper);
sphereActor->SetOrientation(90,0,0);
vtkProperty* ControlProps=vtkProperty::New();
ControlProps->SetColor(r,g,b);




ControlProps->SetOpacity(1.0); 
sphereActor-> SetProperty (ControlProps);
sphereActor->AddPosition(x,y,z);
return sphereActor;
};





vtkAssembly *SpehereActorsForXYZFile(char *FileName, float r, float g, float b )
{

  vtkAssembly  *assembly = vtkAssembly::New();

  string line;
  cout << "Use File:" << FileName << endl;
  ifstream VTK_File (FileName);
  if (!VTK_File.is_open())
   {
     cout << "Error reading File exit!" << endl;
     exit (1);
   };
  int counter=0;
  while ( VTK_File.good() )
    {
      getline (VTK_File,line);
      //kind FileName r g b opacity
      char kind[100];
      char FileName[1024];
      float x,y,z;
      int nrRead=sscanf (line.c_str(),"%f %f %f", &x, &y, &z);
      
    if (nrRead==3)
      {
	r=r+0.05;
	b=b+0.05;
	g=g+0.05;
	

	cout << "Position:" << x << " " << y << " " << z << endl;
	vtkActor* single=SphereActorForPositionAndColor(x,y,z,r,g,b);
	assembly->AddPart(single);
  
      };

    };

  return assembly;

}






list<vtkActor*> ListSpehereActorsForXYZFile(char *FileName, float r, float g, float b )
{

  list<vtkActor*> assembly;

  string line;
  cout << "Use File:" << FileName << endl;
  ifstream VTK_File (FileName);
  if (!VTK_File.is_open())
   {
     cout << "Error reading File exit!" << endl;
     exit (1);
   };
  
  while ( VTK_File.good() )
    {
      getline (VTK_File,line);
      //kind FileName r g b opacity
      char kind[100];
      char FileName[1024];
      float x,y,z;
      int nrRead=sscanf (line.c_str(),"%f %f %f", &x, &y, &z);
    if (nrRead==3)
      {
	cout << "Position:" << x << " " << y << " " << z << endl;
	r=r+0.05;g=g+0.05;b=b+0.05;
	vtkActor* single=SphereActorForPositionAndColor(x,y,z,r,g,b);
	assembly.push_back(single);
  
      };

    };

  return assembly;

}




vtkActor* SphereActorForPositionAndColorAndRadius(float x, float y, float z, float r, float g, float b, float R )
{

 
vtkSphereSource *sphere= vtkSphereSource::New();
sphere->SetRadius(R);
vtkPolyDataMapper *sphereMapper= vtkPolyDataMapper::New();
sphereMapper->SetInput(sphere->GetOutput());
vtkActor *sphereActor=vtkActor::New();
sphereActor->SetMapper(sphereMapper);
sphereActor->SetOrientation(90,0,0);
vtkProperty* ControlProps=vtkProperty::New();
ControlProps->SetColor(r,g,b);
//ControlProps->SetRepresentationToWireframe( );



ControlProps->SetOpacity(1.0); 
sphereActor-> SetProperty (ControlProps);
sphereActor->AddPosition(x,y,z);
return sphereActor;
};





vtkActor* TubeActorForStartStopPositionAndColor(float sx, float sy, float sz,float x, float y, float z, float r, float g, float b )
{
vtkActor *tubeActor=vtkActor::New();

vtkSmartPointer<vtkLineSource> lineSource = 
    vtkSmartPointer<vtkLineSource>::New();
  lineSource->SetPoint1(sx, sy, sz);
  lineSource->SetPoint2(x, y, z);
 
  // Create a mapper and actor
  vtkSmartPointer<vtkPolyDataMapper> lineMapper = 
    vtkSmartPointer<vtkPolyDataMapper>::New();
  lineMapper->SetInputConnection(lineSource->GetOutputPort());
  vtkSmartPointer<vtkActor> lineActor = 
    vtkSmartPointer<vtkActor>::New();
  lineActor->GetProperty()->SetColor(r,g,b); // Give some color to the line
  lineActor->SetMapper(lineMapper);
 
  // Create a tube (cylinder) around the line
  vtkSmartPointer<vtkTubeFilter> tubeFilter = 
    vtkSmartPointer<vtkTubeFilter>::New();
  tubeFilter->SetInputConnection(lineSource->GetOutputPort());
  tubeFilter->SetRadius(0.5); //default is .5
  tubeFilter->SetNumberOfSides(25);
  tubeFilter->Update();
 
  // Create a mapper and actor
  vtkSmartPointer<vtkPolyDataMapper> tubeMapper = 
    vtkSmartPointer<vtkPolyDataMapper>::New();
  tubeMapper->SetInputConnection(tubeFilter->GetOutputPort());
  tubeActor->GetProperty()->SetOpacity(0.5); //Make the tube have some transparency.
  tubeActor->SetMapper(tubeMapper);

  vtkProperty* ControlProps=vtkProperty::New();
 ControlProps->SetColor(r,g,b);
 ControlProps->SetOpacity(1.0); 
tubeActor-> SetProperty (ControlProps);

return tubeActor;
};





























